@@ -19,9 +19,9 @@
 
 #include "model_manager.h"
 #include "utils.h"
+#include "create_state_model.h"
 
 #include "r_interface/boom_r_tools.hpp"
-#include "r_interface/create_state_model.hpp"
 #include "r_interface/handle_exception.hpp"
 #include "r_interface/list_io.hpp"
 #include "r_interface/print_R_timestamp.hpp"
@@ -32,10 +32,9 @@
 #include "cpputil/ThreadTools.hpp"
 
 extern "C" {
-  using BOOM::Vector;
-  using BOOM::Ptr;
-  using BOOM::bsts::ModelManager;
-  using BOOM::RCheckInterrupt;
+  using namespace BOOM;
+  using namespace BOOM::RInterface;
+  using namespace BOOM::bsts;
   using std::endl;
 
   SEXP analysis_common_r_fit_bsts_model_(
@@ -51,7 +50,7 @@ extern "C" {
     BOOM::RErrorReporter error_reporter;
     BOOM::RMemoryProtector protector;
     try {
-      BOOM::RInterface::seed_rng_from_R(r_seed);
+      seed_rng_from_R(r_seed);
       BOOM::RListIoManager io_manager;
       std::string family = BOOM::ToString(r_family);
       int xdim = 0;
@@ -59,14 +58,13 @@ extern "C" {
       if (!Rf_isNull(r_predictors)) {
         xdim = Rf_ncols(r_predictors);
       }
-      std::unique_ptr<ModelManager> model_manager(ModelManager::Create(
-          family, xdim));
-      Ptr<BOOM::ScalarStateSpaceModelBase> model(model_manager->CreateModel(
+      std::unique_ptr<ScalarModelManager> model_manager(
+          ScalarModelManager::Create(family, xdim));
+      Ptr<BOOM::Model> model(model_manager->CreateModel(
           r_data_list,
           r_state_specification,
           r_prior,
           r_options,
-          nullptr,
           &io_manager));
 
       // Do one posterior sampling step before getting ready to write.  This
@@ -120,9 +118,9 @@ extern "C" {
       }
       return ans;
     } catch (std::exception &e) {
-      BOOM::RInterface::handle_exception(e);
+      handle_exception(e);
     } catch (...) {
-      BOOM::RInterface::handle_unknown_exception();
+      handle_unknown_exception();
     }
     return R_NilValue;
   }
@@ -130,39 +128,36 @@ extern "C" {
   // Returns the posterior predictive distribution of a model forecast
   // over a specified forecast period.
   // Args:
-  //   r_bsts_object: The object on which the predictions are to be
-  //     based, which was returned by the original call to bsts.
-  //   r_prediction_data: An R list containing any additional data
-  //     needed to make the prediction.  For simple state space models
-  //     this is just an integer giving the time horizon over which to
-  //     predict.  For models containing a regression component it
-  //     contains the future values of the X's.  For binomial (or
-  //     Poisson) models it contains a sequence of future trial counts
-  //     (or exposures).
-  //   r_burn: An integer giving the number of burn-in iterations to
-  //     discard.  Negative numbers will be treated as zero.  Numbers
-  //     greater than the number of MCMC iterations will raise an error.
-  //   r_observed_data: An R list containing the observed data on which
-  //     to base the prediction.  In typical cases this should be
-  //     R_NilValue (R's NULL) signaling that the observed data should
-  //     be taken from r_bsts_object.  However, if additional data have
-  //     been observed since the model was trained, or if the model is
-  //     being used to predict data that were part of the training set,
-  //     or some other application other than making predictions
-  //     starting from one time period after the training data ends,
-  //     then one can use this argument to pass the "training data" on
-  //     which the predictions should be based.  If this argument is
-  //     used, then the Kalman filter will be run over the supplied
-  //     data, which is expensive.  If this argument is left as
-  //     R_NilValue (NULL) then the "final.state" element of
-  //     r_bsts_object will be used as the basis for future predictions,
-  //     which is a computational savings over filtering from scratch.
+  //   r_bsts_object: The object on which the predictions are to be based, which
+  //     was returned by the original call to bsts.
+  //   r_prediction_data: An R list containing any additional data needed to
+  //     make the prediction.  For simple state space models this is just an
+  //     integer giving the time horizon over which to predict.  For models
+  //     containing a regression component it contains the future values of the
+  //     X's.  For binomial (or Poisson) models it contains a sequence of future
+  //     trial counts (or exposures).
+  //   r_burn: An integer giving the number of burn-in iterations to discard.
+  //     Negative numbers will be treated as zero.  Numbers greater than the
+  //     number of MCMC iterations will raise an error.
+  //   r_observed_data: An R list containing the observed data on which to base
+  //     the prediction.  In typical cases this should be R_NilValue (R's NULL)
+  //     signaling that the observed data should be taken from r_bsts_object.
+  //     However, if additional data have been observed since the model was
+  //     trained, or if the model is being used to predict data that were part
+  //     of the training set, or some other application other than making
+  //     predictions starting from one time period after the training data ends,
+  //     then one can use this argument to pass the "training data" on which the
+  //     predictions should be based.  If this argument is used, then the Kalman
+  //     filter will be run over the supplied data, which is expensive.  If this
+  //     argument is left as R_NilValue (NULL) then the "final.state" element of
+  //     r_bsts_object will be used as the basis for future predictions, which
+  //     is a computational savings over filtering from scratch.
   //
   // Returns:
-  //   An R matrix containing draws from the posterior predictive
-  //   distribution.  Rows of the matrix correspond to MCMC iterations,
-  //   and columns to time points.  The matrix will have 'burn' fewer
-  //   rows than the number of MCMC iterations in r_bsts_object.
+  //   An R matrix containing draws from the posterior predictive distribution.
+  //   Rows of the matrix correspond to MCMC iterations, and columns to time
+  //   points.  The matrix will have 'burn' fewer rows than the number of MCMC
+  //   iterations in r_bsts_object.
   SEXP analysis_common_r_predict_bsts_model_(
       SEXP r_bsts_object,
       SEXP r_prediction_data,
@@ -170,18 +165,18 @@ extern "C" {
       SEXP r_observed_data,
       SEXP r_seed) {
     try {
-      BOOM::RInterface::seed_rng_from_R(r_seed);
-      std::unique_ptr<ModelManager> model_manager(
-          ModelManager::Create(r_bsts_object));
+      seed_rng_from_R(r_seed);
+      std::unique_ptr<ScalarModelManager> model_manager(
+          ScalarModelManager::Create(r_bsts_object));
       return BOOM::ToRMatrix(model_manager->Forecast(
           r_bsts_object,
           r_prediction_data,
           r_burn,
           r_observed_data));
     } catch (std::exception &e) {
-      BOOM::RInterface::handle_exception(e);
+      handle_exception(e);
     } catch (...) {
-      BOOM::RInterface::handle_unknown_exception();
+      handle_unknown_exception();
     }
     return R_NilValue;
   }
@@ -190,32 +185,34 @@ extern "C" {
   // training data or a set of holdout data.
   //
   // Args:
-  //   r_bsts_object: The object on which the predictions are to be
-  //     based, which was returned by the original call to bsts.
+  //   r_bsts_object: The object on which the predictions are to be based, which
+  //     was returned by the original call to bsts.
   //   r_cutpoints: A set of integers ranging from 1 to
-  //     bsts.object$number.of.time.points.  One bsts model run is needed for each
-  //     cutpoint, using data up to that cutpoint.
+  //     bsts.object$number.of.time.points.  One bsts model run is needed for
+  //     each cutpoint, using data up to that cutpoint.
   //
   // Returns:
   //    A list of R matrices with rows corresponding to MCMC draws and columns
   //    corresponding to time.
   SEXP analysis_common_r_bsts_one_step_prediction_errors_(
       SEXP r_bsts_object,
-      SEXP r_cutpoints) {
+      SEXP r_cutpoints,
+      SEXP r_standardize) {
     try {
       std::vector<int> cutpoints = BOOM::ToIntVector(r_cutpoints, true);
       std::vector<BOOM::Matrix> prediction_errors(cutpoints.size());
-
+      bool standardize = Rf_asLogical(r_standardize);
+      
       std::vector<std::future<void>> futures;
       int desired_threads = std::min<int>(
           cutpoints.size(), std::thread::hardware_concurrency() - 1);
       BOOM::ThreadWorkerPool pool;
       pool.add_threads(desired_threads);
       for (int i = 0; i < cutpoints.size(); ++i) {
-        std::unique_ptr<ModelManager> model_manager(
-            ModelManager::Create(r_bsts_object));
+        std::unique_ptr<ScalarModelManager> model_manager(
+            ScalarModelManager::Create(r_bsts_object));
         futures.emplace_back(pool.submit(model_manager->CreateHoldoutSampler(
-            r_bsts_object, cutpoints[i], &prediction_errors[i])));
+            r_bsts_object, cutpoints[i], standardize, &prediction_errors[i])));
       }
       for (int i = 0; i < futures.size(); ++i) {
         futures[i].get();
@@ -228,9 +225,9 @@ extern "C" {
       }
       return ans;
     } catch (std::exception &e) {
-      BOOM::RInterface::handle_exception(e);
+      handle_exception(e);
     } catch (...) {
-      BOOM::RInterface::handle_unknown_exception();
+      handle_unknown_exception();
     }
     return R_NilValue;
   }
